//#include "motor_esp32cam.h"
#include "esp_camera.h"
#include <WiFi.h>

#include "bt_init_wifi.h"

//
// WARNING!!! Make sure that you have either selected ESP32 Wrover Module,
//            or another board which has PSRAM enabled
//

// Select camera model
//#define CAMERA_MODEL_WROVER_KIT
//#define CAMERA_MODEL_ESP_EYE
//#define CAMERA_MODEL_M5STACK_PSRAM
//#define CAMERA_MODEL_M5STACK_WIDE
#define CAMERA_MODEL_AI_THINKER

#include "camera_pins.h"

const char* ssid = "****";
const char* password = "*****";

void startCameraServer();

void setup() {
  Serial.begin(115200);
  Serial.setDebugOutput(true);
  //  Serial.println();

  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;
  //init with high specs to pre-allocate larger buffers
  if (psramFound()) {
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }

#if defined(CAMERA_MODEL_ESP_EYE)
  pinMode(13, INPUT_PULLUP);
  pinMode(14, INPUT_PULLUP);
#endif

  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    return;
  }

  sensor_t * s = esp_camera_sensor_get();
  //initial sensors are flipped vertically and colors are a bit saturated
  if (s->id.PID == OV3660_PID) {
    s->set_vflip(s, 1);//flip it back
    s->set_brightness(s, 1);//up the blightness just a bit
    s->set_saturation(s, -2);//lower the saturation
  }
  //drop down frame size for higher initial frame rate
  s->set_framesize(s, FRAMESIZE_QVGA);

#if defined(CAMERA_MODEL_M5STACK_WIDE)
  s->set_vflip(s, 1);
  s->set_hmirror(s, 1);
#endif

  int statistics = 0;
  long time_start;


#if defined(CAMERA_MODEL_AI_THINKER)
#define FLASH_GPIO 4
#define BUTTON_GPIO 2
#endif
#if defined(CAMERA_MODEL_AI_THINKER)
  pinMode(FLASH_GPIO, OUTPUT);
  pinMode(BUTTON_GPIO, INPUT_PULLUP); //16 GPIO - под кнопку сброса FLASH памяти
#endif
  if (btEN_iniSET()) // Если во флэш памяти есть данные по WiFi
  {
    WiFi.setHostname("HomeBot");
    //WiFi.begin(ssid, password);//,13);
    Serial.print("to_con_wifi_nam="); Serial.println(wifi_nam);
    Serial.print("to_con_wifi_passwd="); Serial.println(wifi_passwd);
    WiFi.begin(wifi_nam, wifi_passwd);
    time_start = 20000 + millis();
    while (WiFi.status() != WL_CONNECTED) //Ждем до 20 секунд
    {
      digitalWrite(FLASH_GPIO, HIGH);
      delay(5);
      digitalWrite(FLASH_GPIO, LOW);
      delay(10);
      Serial.print(".");

      if (time_start < millis()) break;
    }
  }
  else // Если во флэш памяти есть данные по WiFi
  {
    time_start = millis();
  }
  Serial.println(".");
  if (time_start <= millis())//Если соединение с WiFi не удалось
  {
    //
    digitalWrite(FLASH_GPIO, HIGH);

    SerialBT.begin("CamDrvBt"); //Bluetooth device name

    //Serial.print(".");
    while (!SerialBT.connected())
    {
      delay(5);
      digitalWrite(FLASH_GPIO, HIGH);
      delay(10);
      digitalWrite(FLASH_GPIO, LOW);
    }

    while (!bt_read_wifi_set())
    {
      delay(5);
      digitalWrite(FLASH_GPIO, HIGH);
      delay(50);
      digitalWrite(FLASH_GPIO, LOW);
      Serial.println("insert WiFi setting from Bluetooth");
    }
    WiFi.setHostname("HomeBot");
    WiFi.begin(wifi_nam, wifi_passwd);

    time_start = 5000 + millis();
    while (WiFi.status() != WL_CONNECTED)
    {
      digitalWrite(FLASH_GPIO, HIGH);
      delay(10);
      digitalWrite(FLASH_GPIO, LOW);
      delay(450);
      Serial.println(".");
      if (time_start < millis()) break;
    }
    if (WiFi.status() == WL_CONNECTED) {
      SerialBT.print(WiFi.localIP());
      digitalWrite(FLASH_GPIO, HIGH);
    }
    else
    {
      SerialBT.println("000.000.000.000");
    }
    delay(1000);
    ESP.restart();
  }
 
  Serial.println("");
  Serial.println("WiFi connected");

  startCameraServer();

  Serial.print("Camera Ready! Use 'http://");
  Serial.print(WiFi.localIP());
  Serial.println("' to connect");
 
 
}

void loop() {
   delay(1000);
  if (!digitalRead(BUTTON_GPIO)) //Очистить из памяти пароль и имя сети
  {
    wifi_set_p.begin("robot_ini", false);
    wifi_set_p.clear();
    wifi_set_p.end();
    digitalWrite(FLASH_GPIO, HIGH);
    delay(1000);
    digitalWrite(FLASH_GPIO, LOW);
    ESP.restart();
  }
}
